@@ -8,18 +8,18 @@ For example, lets say we have two filesystems containing config files and resour
 
    [config_fs]
    |-- config.cfg
-   `-- defaults.cfg 
+   `-- defaults.cfg
 
    [resources_fs]
    |-- images
    |   |-- logo.jpg
-   |   `-- photo.jpg 
+   |   `-- photo.jpg
    `-- data.dat
 
 We can combine these filesystems in to a single filesystem with the following code::
 
     from fs.mountfs import MountFS
-    combined_fs = MountFS
+    combined_fs = MountFS()
     combined_fs.mountdir('config', config_fs)
     combined_fs.mountdir('resources', resources_fs)
 
@@ -31,7 +31,7 @@ This will create a single filesystem where paths under `config` map to `config_f
     |   `-- defaults.cfg
     `-- resources
         |-- images
-        |   |-- logo.jpg    
+        |   |-- logo.jpg
         |   `-- photo.jpg
         `-- data.dat
 
@@ -39,25 +39,29 @@ Now both filesystems can be accessed with the same path structure::
 
     print combined_fs.getcontents('/config/defaults.cfg')
     read_jpg(combined_fs.open('/resources/images/logo.jpg')
-    
+
 """
 
 from fs.base import *
 from fs.errors import *
 from fs.path import *
 from fs import _thread_synchronize_default
+from fs import iotools
 
 
 class DirMount(object):
     def __init__(self, path, fs):
         self.path = path
-        self.fs = fs        
+        self.fs = fs
 
     def __str__(self):
-        return "Mount point: <%s,%s>" % (self.path,self.fs,)
-    __repr__ = __str__
+        return "<DirMount %s, %s>" % (self.path, self.fs)
+
+    def __repr__(self):
+        return "<DirMount %s, %s>" % (self.path, self.fs)
+
     def __unicode__(self):
-        return unicode(str(self))
+        return u"<DirMount %s, %s>" % (self.path, self.fs)
 
 
 class FileMount(object):
@@ -74,7 +78,7 @@ class MountFS(FS):
     _meta = { 'virtual': True,
               'read_only' : False,
               'unicode_paths' : True,
-              'case_insensitive_paths' : False,              
+              'case_insensitive_paths' : False,
               }
 
     DirMount = DirMount
@@ -83,7 +87,7 @@ class MountFS(FS):
     def __init__(self, auto_close=True, thread_synchronize=_thread_synchronize_default):
         self.auto_close = auto_close
         super(MountFS, self).__init__(thread_synchronize=thread_synchronize)
-        self.mount_tree = PathMap()        
+        self.mount_tree = PathMap()
 
     def __str__(self):
         return "<%s [%s]>" % (self.__class__.__name__,self.mount_tree.items(),)
@@ -91,7 +95,7 @@ class MountFS(FS):
     __repr__ = __str__
 
     def __unicode__(self):
-        return unicode(self.__str__())
+        return u"<%s [%s]>" % (self.__class__.__name__,self.mount_tree.items(),)
 
     def _delegate(self, path):
         path = abspath(normpath(path))
@@ -125,11 +129,11 @@ class MountFS(FS):
     def close(self):
         # Explicitly closes children if requested
         if self.auto_close:
-            for mount in self.mount_tree.itervalues():            
+            for mount in self.mount_tree.itervalues():
                 mount.fs.close()
         # Free references (which may incidently call the close method of the child filesystems)
-        self.mount_tree.clear()        
-        super(MountFS, self).close()            
+        self.mount_tree.clear()
+        super(MountFS, self).close()
 
     def getsyspath(self, path, allow_none=False):
         fs, _mount_path, delegate_path = self._delegate(path)
@@ -139,7 +143,7 @@ class MountFS(FS):
             else:
                 raise NoSysPathError(path=path)
         return fs.getsyspath(delegate_path, allow_none=allow_none)
- 
+
     def getpathurl(self, path, allow_none=False):
         fs, _mount_path, delegate_path = self._delegate(path)
         if fs is self or fs is None:
@@ -157,16 +161,19 @@ class MountFS(FS):
                 return "Mount dir"
             else:
                 return "Mounted file"
-        return "Mounted dir, maps to path %s on %s" % (delegate_path, str(fs))
+        return "Mounted dir, maps to path %s on %s" % (abspath(delegate_path) or '/', str(fs))
 
     @synchronize
     def isdir(self, path):
         fs, _mount_path, delegate_path = self._delegate(path)
         if fs is None:
+            path = normpath(path)
+            if path in ("/", ""):
+                return True
             return False
         if fs is self:
-            object = self.mount_tree.get(path, None)
-            return not isinstance(object,MountFS.FileMount)
+            obj = self.mount_tree.get(path, None)
+            return not isinstance(obj, MountFS.FileMount)
         return fs.isdir(delegate_path)
 
     @synchronize
@@ -175,12 +182,14 @@ class MountFS(FS):
         if fs is None:
             return False
         if fs is self:
-            object = self.mount_tree.get(path, None)
-            return isinstance(object,MountFS.FileMount)
+            obj = self.mount_tree.get(path, None)
+            return isinstance(obj, MountFS.FileMount)
         return fs.isfile(delegate_path)
 
     @synchronize
     def exists(self, path):
+        if path in ("/", ""):
+            return True
         fs, _mount_path, delegate_path = self._delegate(path)
         if fs is None:
             return False
@@ -193,9 +202,11 @@ class MountFS(FS):
         fs, _mount_path, delegate_path = self._delegate(path)
 
         if fs is None:
-            raise ResourceNotFoundError(path)
+            if path in ("/", ""):
+                return []
+            raise ResourceNotFoundError("path")
 
-        if fs is self:
+        elif fs is self:
             paths = self.mount_tree.names(path)
             return self._listdir_helper(path,
                                         paths,
@@ -235,6 +246,8 @@ class MountFS(FS):
         fs, _mount_path, delegate_path = self._delegate(path)
 
         if fs is None:
+            if path in ("/", ""):
+                return
             raise ResourceNotFoundError(path)
 
         if fs is self:
@@ -272,24 +285,23 @@ class MountFS(FS):
                 else:
                     yield mkpath(p)
 
-
     @synchronize
     def makedir(self, path, recursive=False, allow_recreate=False):
         fs, _mount_path, delegate_path = self._delegate(path)
         if fs is self or fs is None:
-            raise UnsupportedError("make directory", msg="Can only makedir for mounted paths" )
+            raise UnsupportedError("make directory", msg="Can only makedir for mounted paths")
         if not delegate_path:
             if allow_recreate:
                 return
-            else:                
+            else:
                 raise DestinationExistsError(path, msg="Can not create a directory that already exists (try allow_recreate=True): %(path)s")
         return fs.makedir(delegate_path, recursive=recursive, allow_recreate=allow_recreate)
 
     @synchronize
-    def open(self, path, mode="r", **kwargs):
-        object = self.mount_tree.get(path, None)
-        if type(object) is MountFS.FileMount:
-            callable = object.open_callable
+    def open(self, path, mode='r', buffering=-1, encoding=None, errors=None, newline=None, line_buffering=False, **kwargs):
+        obj = self.mount_tree.get(path, None)
+        if type(obj) is MountFS.FileMount:
+            callable = obj.open_callable
             return callable(path, mode, **kwargs)
 
         fs, _mount_path, delegate_path = self._delegate(path)
@@ -300,20 +312,24 @@ class MountFS(FS):
         return fs.open(delegate_path, mode, **kwargs)
 
     @synchronize
-    def setcontents(self, path, data, chunk_size=64*1024):
-        object = self.mount_tree.get(path, None)
-        if type(object) is MountFS.FileMount:
-            return super(MountFS,self).setcontents(path, data, chunk_size=chunk_size)
+    def setcontents(self, path, data=b'', encoding=None, errors=None, chunk_size=64*1024):
+        obj = self.mount_tree.get(path, None)
+        if type(obj) is MountFS.FileMount:
+            return super(MountFS, self).setcontents(path,
+                                                    data,
+                                                    encoding=encoding,
+                                                    errors=errors,
+                                                    chunk_size=chunk_size)
         fs, _mount_path, delegate_path = self._delegate(path)
         if fs is self or fs is None:
             raise ParentDirectoryMissingError(path)
-        return fs.setcontents(delegate_path, data, chunk_size)
+        return fs.setcontents(delegate_path, data, encoding=encoding, errors=errors, chunk_size=chunk_size)
 
     @synchronize
     def createfile(self, path, wipe=False):
-        object = self.mount_tree.get(path, None)
-        if type(object) is MountFS.FileMount:
-            return super(MountFS,self).createfile(path, wipe=wipe)
+        obj = self.mount_tree.get(path, None)
+        if type(obj) is MountFS.FileMount:
+            return super(MountFS, self).createfile(path, wipe=wipe)
         fs, _mount_path, delegate_path = self._delegate(path)
         if fs is self or fs is None:
             raise ParentDirectoryMissingError(path)
@@ -329,6 +345,8 @@ class MountFS(FS):
     @synchronize
     def removedir(self, path, recursive=False, force=False):
         path = normpath(path)
+        if path in ('', '/'):
+            raise RemoveRootError(path)
         fs, _mount_path, delegate_path = self._delegate(path)
         if fs is self or fs is None:
             raise ResourceInvalidError(path, msg="Can not removedir for an un-mounted path")
@@ -392,33 +410,43 @@ class MountFS(FS):
     @synchronize
     def mountdir(self, path, fs):
         """Mounts a host FS object on a given path.
-        
+
         :param path: A path within the MountFS
-        :param fs: A filesystem object to mount        
+        :param fs: A filesystem object to mount
 
         """
+        path = abspath(normpath(path))
         self.mount_tree[path] = MountFS.DirMount(path, fs)
     mount = mountdir
 
     @synchronize
     def mountfile(self, path, open_callable=None, info_callable=None):
         """Mounts a single file path.
-        
+
         :param path: A path within the MountFS
-        :param open_callable: A callable that returns a file-like object
-        :param info_callable: A callable that returns a dictionary with information regarding the file-like object
-        
+        :param open_callable: A callable that returns a file-like object,
+            `open_callable` should have the same signature as :py:meth:`~fs.base.FS.open`
+        :param info_callable: A callable that returns a dictionary with information regarding the file-like object,
+            `info_callable` should have the same signagture as :py:meth:`~fs.base.FS.getinfo`
+
         """
-        self.mount_tree[path] = MountFS.FileMount(path, callable, info_callable)
+        self.mount_tree[path] = MountFS.FileMount(path, open_callable, info_callable)
 
     @synchronize
     def unmount(self, path):
         """Unmounts a path.
 
         :param path: Path to unmount
+        :return: True if a path was unmounted, False if the path was already unmounted
+        :rtype: bool
 
         """
-        del self.mount_tree[path]
+        try:
+            del self.mount_tree[path]
+        except KeyError:
+            return False
+        else:
+            return True
 
     @synchronize
     def settimes(self, path, accessed_time=None, modified_time=None):
@@ -439,6 +467,8 @@ class MountFS(FS):
         fs, _mount_path, delegate_path = self._delegate(path)
 
         if fs is None:
+            if path in ("/", ""):
+                return {}
             raise ResourceNotFoundError(path)
 
         if fs is self:
@@ -473,6 +503,8 @@ class MountFS(FS):
         path = normpath(path)
         fs, _mount_path, delegate_path = self._delegate(path)
         if fs is None:
+            if path in ("/", ""):
+                return default
             raise ResourceNotFoundError(path)
         if fs is self:
             return default
