@@ -12,10 +12,16 @@ import time
 from os.path import abspath
 import urllib
 
+from six import PY3
+
+
 try:
-    from pyftpdlib import ftpserver
+    from pyftpdlib.authorizers import DummyAuthorizer
+    from pyftpdlib.handlers import FTPHandler
+    from pyftpdlib.servers import FTPServer
 except ImportError:
-    raise ImportError("Requires pyftpdlib <http://code.google.com/p/pyftpdlib/>")
+    if not PY3:
+        raise ImportError("Requires pyftpdlib <http://code.google.com/p/pyftpdlib/>")
 
 from fs.path import *
 
@@ -24,6 +30,8 @@ from fs import ftpfs
 ftp_port = 30000
 class TestFTPFS(unittest.TestCase, FSTestCases, ThreadingTestCases):
 
+    __test__ = not PY3
+
     def setUp(self):
         global ftp_port
         ftp_port += 1
@@ -41,11 +49,11 @@ class TestFTPFS(unittest.TestCase, FSTestCases, ThreadingTestCases):
                                             file_path,
                                             self.temp_dir,
                                             use_port],
-                                           stdout=subprocess.PIPE, 
+                                           stdout=subprocess.PIPE,
                                            env=env)
         # Block until the server writes a line to stdout
         self.ftp_server.stdout.readline()
-        
+
         # Poll until a connection can be made
         start_time = time.time()
         while time.time() - start_time < 5:
@@ -60,14 +68,14 @@ class TestFTPFS(unittest.TestCase, FSTestCases, ThreadingTestCases):
         else:
             # Avoid a possible infinite loop
             raise Exception("Unable to connect to ftp server")
-        
+
         self.fs = ftpfs.FTPFS('127.0.0.1', 'user', '12345', dircache=True, port=use_port, timeout=5.0)
         self.fs.cache_hint(True)
 
 
     def tearDown(self):
-        #self.ftp_server.terminate()        
-        if sys.platform == 'win32':        
+        #self.ftp_server.terminate()
+        if sys.platform == 'win32':
             os.popen('TASKKILL /PID '+str(self.ftp_server.pid)+' /F')
         else:
             os.system('kill '+str(self.ftp_server.pid))
@@ -83,21 +91,21 @@ if __name__ == "__main__":
 
     # Run an ftp server that exposes a given directory
     import sys
-    authorizer = ftpserver.DummyAuthorizer()    
+    authorizer = DummyAuthorizer()
     authorizer.add_user("user", "12345", sys.argv[1], perm="elradfmw")
     authorizer.add_anonymous(sys.argv[1])
 
-    def nolog(*args):
-        pass
-    ftpserver.log = nolog
-    ftpserver.logline = nolog
+    #def nolog(*args):
+    #    pass
+    #ftpserver.log = nolog
+    #ftpserver.logline = nolog
 
-    handler = ftpserver.FTPHandler
+    handler = FTPHandler
     handler.authorizer = authorizer
     address = ("127.0.0.1", int(sys.argv[2]))
     #print address
 
-    ftpd = ftpserver.FTPServer(address, handler)
+    ftpd = FTPServer(address, handler)
 
     sys.stdout.write('serving\n')
     sys.stdout.flush()
