@@ -22,24 +22,32 @@ from fs.tempfs import TempFS
 from fs.path import *
 from fs.local_functools import wraps
 
+from six import PY3, b
+
 
 class RemoteTempFS(TempFS):
     """
         Simple filesystem implementing setfilecontents
         for RemoteFileBuffer tests
     """
-    def open(self, path, mode='rb', write_on_flush=True):
+    def __repr__(self):
+        return '<RemoteTempFS: %s>' % self._temp_dir
+
+    def open(self, path, mode='rb', write_on_flush=True, **kwargs):
         if 'a' in mode or 'r' in mode or '+' in mode:
-            f = super(RemoteTempFS, self).open(path, 'rb')
+            f = super(RemoteTempFS, self).open(path, mode='rb', **kwargs)
             f = TellAfterCloseFile(f)
         else:
             f = None
-        
-        return RemoteFileBuffer(self, path, mode, f,
-                                      write_on_flush=write_on_flush)
-        
-    def setcontents(self, path, data, chunk_size=64*1024):
-        f = super(RemoteTempFS, self).open(path, 'wb')
+
+        return RemoteFileBuffer(self,
+                                path,
+                                mode,
+                                f,
+                                write_on_flush=write_on_flush)
+
+    def setcontents(self, path, data, encoding=None, errors=None, chunk_size=64*1024):
+        f = super(RemoteTempFS, self).open(path, 'wb', encoding=encoding, errors=errors, chunk_size=chunk_size)
         if getattr(data, 'read', False):
             f.write(data.read())
         else:
@@ -50,7 +58,7 @@ class RemoteTempFS(TempFS):
 class TellAfterCloseFile(object):
     """File-like object that allows calling tell() after it's been closed."""
 
-    def __init__(self,file):
+    def __init__(self, file):
         self._finalpos = None
         self.file = file
 
@@ -64,50 +72,49 @@ class TellAfterCloseFile(object):
             return self._finalpos
         return self.file.tell()
 
-    def __getattr__(self,attr):        
-        return getattr(self.file,attr)
+    def __getattr__(self, attr):
+        return getattr(self.file, attr)
 
 
 class TestRemoteFileBuffer(unittest.TestCase, FSTestCases, ThreadingTestCases):
     class FakeException(Exception): pass
-    
+
     def setUp(self):
         self.fs = RemoteTempFS()
         self.original_setcontents = self.fs.setcontents
 
-    def tearDown(self):        
+    def tearDown(self):
         self.fs.close()
         self.fakeOff()
 
-    def fake_setcontents(self, path, content='', chunk_size=16*1024):
+    def fake_setcontents(self, path, content=b(''), chunk_size=16*1024):
         ''' Fake replacement for RemoteTempFS setcontents() '''
         raise self.FakeException("setcontents should not be called here!")
-    
+
     def fakeOn(self):
         '''
             Turn on fake_setcontents(). When setcontents on RemoteTempFS
             is called, FakeException is raised and nothing is stored.
         '''
-        self.original_setcontents = self.fs.setcontents
         self.fs.setcontents = self.fake_setcontents
-        
+
     def fakeOff(self):
         ''' Switch off fake_setcontents(). '''
         self.fs.setcontents = self.original_setcontents
-        
+
     def test_ondemand(self):
         '''
             Tests on-demand loading of remote content in RemoteFileBuffer
-        '''    
-        contents = "Tristatricettri stribrnych strikacek strikalo" + \
-                   "pres tristatricettri stribrnych strech."
+        '''
+        contents = b("Tristatricettri stribrnych strikacek strikalo") + \
+                   b("pres tristatricettri stribrnych strech.")
         f = self.fs.open('test.txt', 'wb')
         f.write(contents)
         f.close()
-        
+
         # During following tests, no setcontents() should be called.
         self.fakeOn()
-        
+
         f = self.fs.open('test.txt', 'rb')
         self.assertEquals(f.read(10), contents[:10])
         f.wrapped_file.seek(0, SEEK_END)
@@ -118,40 +125,40 @@ class TestRemoteFileBuffer(unittest.TestCase, FSTestCases, ThreadingTestCases):
         f.seek(0, SEEK_END)
         self.assertEquals(f._rfile.tell(), len(contents))
         f.close()
-        
+
         f = self.fs.open('test.txt', 'ab')
         self.assertEquals(f.tell(), len(contents))
         f.close()
-        
+
         self.fakeOff()
-        
+
         # Writing over the rfile edge
         f = self.fs.open('test.txt', 'wb+')
         self.assertEquals(f.tell(), 0)
         f.seek(len(contents) - 5)
-        # Last 5 characters not loaded from remote file  
+        # Last 5 characters not loaded from remote file
         self.assertEquals(f._rfile.tell(), len(contents) - 5)
         # Confirm that last 5 characters are still in rfile buffer
         self.assertEquals(f._rfile.read(), contents[-5:])
         # Rollback position 5 characters before eof
         f._rfile.seek(len(contents[:-5]))
         # Write 10 new characters (will make contents longer for 5 chars)
-        f.write(u'1234567890')
+        f.write(b('1234567890'))
         f.flush()
         # We are on the end of file (and buffer not serve anything anymore)
-        self.assertEquals(f.read(), '')
+        self.assertEquals(f.read(), b(''))
         f.close()
-        
+
         self.fakeOn()
-        
+
         # Check if we wrote everything OK from
         # previous writing over the remote buffer edge
         f = self.fs.open('test.txt', 'rb')
-        self.assertEquals(f.read(), contents[:-5] + u'1234567890')
+        self.assertEquals(f.read(), contents[:-5] + b('1234567890'))
         f.close()
 
         self.fakeOff()
-    
+
     def test_writeonflush(self):
         '''
             Test 'write_on_flush' switch of RemoteFileBuffer.
@@ -161,31 +168,31 @@ class TestRemoteFileBuffer(unittest.TestCase, FSTestCases, ThreadingTestCases):
         '''
         self.fakeOn()
         f = self.fs.open('test.txt', 'wb', write_on_flush=True)
-        f.write('Sample text')
+        f.write(b('Sample text'))
         self.assertRaises(self.FakeException, f.flush)
-        f.write('Second sample text')
+        f.write(b('Second sample text'))
         self.assertRaises(self.FakeException, f.close)
         self.fakeOff()
         f.close()
         self.fakeOn()
-        
+
         f = self.fs.open('test.txt', 'wb', write_on_flush=False)
-        f.write('Sample text')
+        f.write(b('Sample text'))
         # FakeException is not raised, because setcontents is not called
         f.flush()
-        f.write('Second sample text')
+        f.write(b('Second sample text'))
         self.assertRaises(self.FakeException, f.close)
         self.fakeOff()
-        
+
     def test_flush_and_continue(self):
         '''
             This tests if partially loaded remote buffer can be flushed
             back to remote destination and opened file is still
-            in good condition.        
+            in good condition.
         '''
-        contents = "Zlutoucky kun upel dabelske ody."
-        contents2 = 'Ententyky dva spaliky cert vyletel z elektriky'
-                
+        contents = b("Zlutoucky kun upel dabelske ody.")
+        contents2 = b('Ententyky dva spaliky cert vyletel z elektriky')
+
         f = self.fs.open('test.txt', 'wb')
         f.write(contents)
         f.close()
@@ -195,19 +202,19 @@ class TestRemoteFileBuffer(unittest.TestCase, FSTestCases, ThreadingTestCases):
         self.assertEquals(f.read(10), contents[:10])
         self.assertEquals(f._rfile.tell(), 10)
         # Write garbage to file to mark it as _changed
-        f.write('x')
+        f.write(b('x'))
         # This should read the rest of file and store file back to again.
         f.flush()
         f.seek(0)
         # Try if we have unocrrupted file locally...
-        self.assertEquals(f.read(), contents[:10] + 'x' + contents[11:])
+        self.assertEquals(f.read(), contents[:10] + b('x') + contents[11:])
         f.close()
-        
+
         # And if we have uncorrupted file also on storage
         f = self.fs.open('test.txt', 'rb')
-        self.assertEquals(f.read(), contents[:10] + 'x' + contents[11:])
+        self.assertEquals(f.read(), contents[:10] + b('x') + contents[11:])
         f.close()
-        
+
         # Now try it again, but write garbage behind edge of remote file
         f = self.fs.open('test.txt', 'rb+')
         self.assertEquals(f.read(10), contents[:10])
@@ -218,12 +225,12 @@ class TestRemoteFileBuffer(unittest.TestCase, FSTestCases, ThreadingTestCases):
         # Try if we have unocrrupted file locally...
         self.assertEquals(f.read(), contents[:10] + contents2)
         f.close()
-        
+
         # And if we have uncorrupted file also on storage
         f = self.fs.open('test.txt', 'rb')
         self.assertEquals(f.read(), contents[:10] + contents2)
         f.close()
-        
+
 
 class TestCacheFS(unittest.TestCase,FSTestCases,ThreadingTestCases):
     """Test simple operation of CacheFS"""
@@ -243,7 +250,7 @@ class TestCacheFS(unittest.TestCase,FSTestCases,ThreadingTestCases):
         self.fs.cache_timeout = None
         try:
             self.assertFalse(self.fs.isfile("hello"))
-            self.wrapped_fs.setcontents("hello","world")
+            self.wrapped_fs.setcontents("hello",b("world"))
             self.assertTrue(self.fs.isfile("hello"))
             self.wrapped_fs.remove("hello")
             self.assertTrue(self.fs.isfile("hello"))
@@ -257,17 +264,17 @@ class TestCacheFS(unittest.TestCase,FSTestCases,ThreadingTestCases):
         self.fs.cache_timeout = None
         try:
             self.assertFalse(self.fs.isfile("hello"))
-            self.wrapped_fs.setcontents("hello","world")
+            self.wrapped_fs.setcontents("hello",b("world"))
             self.assertTrue(self.fs.isfile("hello"))
             self.wrapped_fs.remove("hello")
             self.assertTrue(self.fs.isfile("hello"))
-            self.wrapped_fs.setcontents("hello","world")
+            self.wrapped_fs.setcontents("hello",b("world"))
             self.assertTrue(self.fs.isfile("hello"))
             self.fs.remove("hello")
             self.assertFalse(self.fs.isfile("hello"))
         finally:
             self.fs.cache_timeout = old_timeout
-         
+
 
 
 class TestConnectionManagerFS(unittest.TestCase,FSTestCases):#,ThreadingTestCases):
@@ -286,7 +293,7 @@ class TestConnectionManagerFS(unittest.TestCase,FSTestCases):#,ThreadingTestCase
 class DisconnectingFS(WrapFS):
     """FS subclass that raises lots of RemoteConnectionErrors."""
 
-    def __init__(self,fs=None):        
+    def __init__(self,fs=None):
         if fs is None:
             fs = TempFS()
         self._connected = True
@@ -315,8 +322,8 @@ class DisconnectingFS(WrapFS):
             time.sleep(random.random()*0.1)
             self._connected = not self._connected
 
-    def setcontents(self, path, contents='', chunk_size=64*1024):
-        return self.wrapped_fs.setcontents(path, contents)
+    def setcontents(self, path, data=b(''), encoding=None, errors=None, chunk_size=64*1024):
+        return self.wrapped_fs.setcontents(path, data, encoding=encoding, errors=errors, chunk_size=chunk_size)
 
     def close(self):
         if not self.closed:
