@@ -13,7 +13,13 @@ import unittest
 from fs.tests import FSTestCases, ThreadingTestCases
 from fs.path import *
 
-from fs import s3fs
+from six import PY3
+try:
+    from fs import s3fs
+except ImportError:
+    raise unittest.SkipTest("s3fs wasn't importable")    
+    
+
 class TestS3FS(unittest.TestCase,FSTestCases,ThreadingTestCases):
 
     #  Disable the tests by default
@@ -21,7 +27,7 @@ class TestS3FS(unittest.TestCase,FSTestCases,ThreadingTestCases):
 
     bucket = "test-s3fs.rfk.id.au"
 
-    def setUp(self):
+    def setUp(self):        
         self.fs = s3fs.S3FS(self.bucket)
         for k in self.fs._s3bukt.list():
             self.fs._s3bukt.delete_key(k)
