@@ -5,37 +5,49 @@ fs.watch
 Change notification support for FS.
 
 This module defines a standard interface for FS subclasses that support change
-notification callbacks.  It also offers some WrapFS subclasses that can 
+notification callbacks.  It also offers some WrapFS subclasses that can
 simulate such an ability on top of an ordinary FS object.
 
 An FS object that wants to be "watchable" must provide the following methods:
 
-  * add_watcher(path,callback,events=None,recursive=True)
+  * ``add_watcher(callback,path="/",events=None,recursive=True)``
 
-      Request that the given callback be executed in response to changes 
+      Request that the given callback be executed in response to changes
       to the given path.  A specific set of change events can be specified.
       This method returns a Watcher object.
 
-  * del_watcher(watcher_or_callback)
+  * ``del_watcher(watcher_or_callback)``
 
       Remove the given watcher object, or any watchers associated with
       the given callback.
 
+
+If you would prefer to read changes from a filesystem in a blocking fashion
+rather than using callbacks, you can use the function 'iter_changes' to obtain
+an iterator over the change events.
+
+
 """
 
+import sys
 import weakref
-import threading 
+import threading
 import Queue
+import traceback
 
 from fs.path import *
 from fs.errors import *
 from fs.wrapfs import WrapFS
 from fs.base import FS
+from fs.filelike import FileWrapper
+
+from six import b
 
 
 class EVENT(object):
     """Base class for change notification events."""
     def __init__(self,fs,path):
+        super(EVENT, self).__init__()
         self.fs = fs
         if path is not None:
             path = abspath(normpath(path))
@@ -47,6 +59,14 @@ class EVENT(object):
     def __unicode__(self):
         return u"<fs.watch.%s object (path='%s') at %s>" % (self.__class__.__name__,self.path,hex(id(self)))
 
+    def clone(self,fs=None,path=None):
+        if fs is None:
+            fs = self.fs
+        if path is None:
+            path = self.path
+        return self.__class__(fs,path)
+
+
 class ACCESSED(EVENT):
     """Event fired when a file's contents are accessed."""
     pass
@@ -61,26 +81,54 @@ class REMOVED(EVENT):
 
 class MODIFIED(EVENT):
     """Event fired when a file or directory is modified."""
-    def __init__(self,fs,path,data_changed=False):
+    def __init__(self,fs,path,data_changed=False, closed=False):
         super(MODIFIED,self).__init__(fs,path)
         self.data_changed = data_changed
+        self.closed = closed
+
+    def clone(self,fs=None,path=None,data_changed=None):
+        evt = super(MODIFIED,self).clone(fs,path)
+        if data_changed is None:
+            data_changed = self.data_changed
+        evt.data_changed = data_changed
+        return evt
 
 class MOVED_DST(EVENT):
     """Event fired when a file or directory is the target of a move."""
-    def __init__(self,fs,path,source):
+    def __init__(self,fs,path,source=None):
         super(MOVED_DST,self).__init__(fs,path)
         if source is not None:
             source = abspath(normpath(source))
         self.source = source
 
+    def __unicode__(self):
+        return u"<fs.watch.%s object (path=%r,src=%r) at %s>" % (self.__class__.__name__,self.path,self.source,hex(id(self)))
+
+    def clone(self,fs=None,path=None,source=None):
+        evt = super(MOVED_DST,self).clone(fs,path)
+        if source is None:
+            source = self.source
+        evt.source = source
+        return evt
+
 class MOVED_SRC(EVENT):
     """Event fired when a file or directory is the source of a move."""
-    def __init__(self,fs,path,destination):
+    def __init__(self,fs,path,destination=None):
         super(MOVED_SRC,self).__init__(fs,path)
         if destination is not None:
             destination = abspath(normpath(destination))
         self.destination = destination
 
+    def __unicode__(self):
+        return u"<fs.watch.%s object (path=%r,dst=%r) at %s>" % (self.__class__.__name__,self.path,self.destination,hex(id(self)))
+
+    def clone(self,fs=None,path=None,destination=None):
+        evt = super(MOVED_SRC,self).clone(fs,path)
+        if destination is None:
+            destination = self.destination
+        evt.destination = destination
+        return evt
+
 class CLOSED(EVENT):
     """Event fired when the filesystem is closed."""
     pass
@@ -131,7 +179,11 @@ class Watcher(object):
                 if event.path != self.path:
                     if dirname(event.path) != self.path:
                         return
-        self.callback(event)
+        try:
+            self.callback(event)
+        except Exception:
+            print >>sys.stderr, "error in FS watcher callback", self.callback
+            traceback.print_exc()
 
 
 class WatchableFSMixin(FS):
@@ -141,6 +193,15 @@ class WatchableFSMixin(FS):
         self._watchers = PathMap()
         super(WatchableFSMixin,self).__init__(*args,**kwds)
 
+    def __getstate__(self):
+        state = super(WatchableFSMixin,self).__getstate__()
+        state.pop("_watchers",None)
+        return state
+
+    def __setstate__(self,state):
+        super(WatchableFSMixin,self).__setstate__(state)
+        self._watchers = PathMap()
+
     def add_watcher(self,callback,path="/",events=None,recursive=True):
         """Add a watcher callback to the FS."""
         w = Watcher(self,callback,path,events,recursive=recursive)
@@ -165,9 +226,14 @@ class WatchableFSMixin(FS):
                 if watcher.callback is callback:
                     yield watcher
 
-    def notify_watchers(self,event_class,path=None,*args,**kwds):
+    def notify_watchers(self,event_or_class,path=None,*args,**kwds):
         """Notify watchers of the given event data."""
-        event = event_class(self,path,*args,**kwds)
+        if isinstance(event_or_class,EVENT):
+            event = event_or_class
+        else:
+            event = event_or_class(self,path,*args,**kwds)
+        if path is None:
+            path = event.path
         if path is None:
             for watchers in self._watchers.itervalues():
                 for watcher in watchers:
@@ -180,50 +246,43 @@ class WatchableFSMixin(FS):
 
 
 
-class WatchedFile(object):
+class WatchedFile(FileWrapper):
     """File wrapper for use with WatchableFS.
 
     This file wrapper provides access to a file opened from a WatchableFS
     instance, and fires MODIFIED events when the file is modified.
     """
 
-    def __init__(self,file,fs,path,mode):
-        self.file = file
+    def __init__(self,file,fs,path,mode=None):
+        super(WatchedFile,self).__init__(file,mode)
         self.fs = fs
         self.path = path
-        self.mode = mode
+        self.was_modified = False
 
-    def __del__(self):
-        self.close()
+    def _write(self,string,flushing=False):
+        self.was_modified = True
+        return super(WatchedFile,self)._write(string,flushing=flushing)
 
-    def __getattr__(self,name):
-        file = self.__dict__['file']
-        a = getattr(file, name)
-        if callable(a):
-            setattr(self,name,a)
-        return a
-
-    def __enter__(self):
-        self.file.__enter__()
-        return self
-
-    def __exit__(self,exc_type,exc_value,traceback):
-        self.close()
-        return False
-
-    def __iter__(self):
-        return iter(self.file)
+    def _truncate(self,size):
+        self.was_modified = True
+        return super(WatchedFile,self)._truncate(size)
 
     def flush(self):
-        self.file.flush()
-        self.fs.notify_watchers(MODIFIED,self.path,True)
+        super(WatchedFile,self).flush()
+        #  Don't bother if python if being torn down
+        if Watcher is not None:
+            if self.was_modified:
+                self.fs.notify_watchers(MODIFIED,self.path,True)
 
     def close(self):
-        self.file.close()
-        self.fs.notify_watchers(MODIFIED,self.path,True)
+        super(WatchedFile,self).close()
+        #  Don't bother if python if being torn down
+        if Watcher is not None:
+            if self.was_modified:
+                self.fs.notify_watchers(MODIFIED,self.path,True)
 
 
-class WatchableFS(WrapFS,WatchableFSMixin):
+class WatchableFS(WatchableFSMixin,WrapFS):
     """FS wrapper simulating watcher callbacks.
 
     This FS wrapper intercepts method calls that modify the underlying FS
@@ -232,24 +291,57 @@ class WatchableFS(WrapFS,WatchableFSMixin):
     that might be made through other interfaces to the same filesystem.
     """
 
-    def __init__(self,*args,**kwds):
-        super(WatchableFS,self).__init__(*args,**kwds)
+    def __init__(self, *args, **kwds):
+        super(WatchableFS, self).__init__(*args, **kwds)
 
     def close(self):
-        super(WatchableFS,self).close()
+        super(WatchableFS, self).close()
         self.notify_watchers(CLOSED)
 
-    def open(self,path,mode="r",**kwargs):
+    def open(self, path, mode='r', buffering=-1, encoding=None, errors=None, newline=None, line_buffering=False, **kwargs):
         existed = self.wrapped_fs.isfile(path)
-        f = super(WatchableFS,self).open(path,mode,**kwargs)
+        f = super(WatchableFS, self).open(path,
+                                          mode=mode,
+                                          buffering=buffering,
+                                          encoding=encoding,
+                                          errors=errors,
+                                          newline=newline,
+                                          line_buffering=line_buffering,
+                                          **kwargs)
+        if not existed:
+            self.notify_watchers(CREATED, path)
+        self.notify_watchers(ACCESSED, path)
+        return WatchedFile(f, self, path, mode)
+
+    def setcontents(self, path, data=b'', encoding=None, errors=None, chunk_size=64*1024):
+        existed = self.wrapped_fs.isfile(path)
+        ret = super(WatchableFS, self).setcontents(path, data=data, encoding=encoding, errors=errors, chunk_size=chunk_size)
+        if not existed:
+            self.notify_watchers(CREATED, path)
+        self.notify_watchers(ACCESSED, path)
+        if data:
+            self.notify_watchers(MODIFIED, path, True)
+        return ret
+
+    def createfile(self, path, wipe=False):
+        existed = self.wrapped_fs.isfile(path)
+        ret = super(WatchableFS, self).createfile(path, wipe=wipe)
         if not existed:
             self.notify_watchers(CREATED,path)
         self.notify_watchers(ACCESSED,path)
-        return WatchedFile(f,self,path,mode)
+        return ret
 
-    def makedir(self,path,*args,**kwds):
+    def makedir(self,path,recursive=False,allow_recreate=False):
         existed = self.wrapped_fs.isdir(path)
-        super(WatchableFS,self).makedir(path,*args,**kwds)
+        try:
+            super(WatchableFS,self).makedir(path,allow_recreate=allow_recreate)
+        except ParentDirectoryMissingError:
+            if not recursive:
+                raise
+            parent = dirname(path)
+            if parent != path:
+                self.makedir(dirname(path),recursive=True,allow_recreate=True)
+            super(WatchableFS,self).makedir(path,allow_recreate=allow_recreate)
         if not existed:
             self.notify_watchers(CREATED,path)
 
@@ -257,17 +349,37 @@ class WatchableFS(WrapFS,WatchableFSMixin):
         super(WatchableFS,self).remove(path)
         self.notify_watchers(REMOVED,path)
 
-    def removedir(self,path,*args,**kwds):
-        super(WatchableFS,self).removedir(path,*args,**kwds)
+    def removedir(self,path,recursive=False,force=False):
+        if not force:
+            for nm in self.listdir(path):
+                raise DirectoryNotEmptyError(path)
+        else:
+            for nm in self.listdir(path,dirs_only=True):
+                try:
+                    self.removedir(pathjoin(path,nm),force=True)
+                except ResourceNotFoundError:
+                    pass
+            for nm in self.listdir(path,files_only=True):
+                try:
+                    self.remove(pathjoin(path,nm))
+                except ResourceNotFoundError:
+                    pass
+        super(WatchableFS,self).removedir(path)
         self.notify_watchers(REMOVED,path)
+        if recursive:
+            parent = dirname(path)
+            while parent and not self.listdir(parent):
+                super(WatchableFS,self).removedir(parent)
+                self.notify_watchers(REMOVED,parent)
+                parent = dirname(parent)
 
     def rename(self,src,dst):
         d_existed = self.wrapped_fs.exists(dst)
         super(WatchableFS,self).rename(src,dst)
         if d_existed:
             self.notify_watchers(REMOVED,dst)
-        self.notify_watchers(MOVED_SRC,src,dst)
         self.notify_watchers(MOVED_DST,dst,src)
+        self.notify_watchers(MOVED_SRC,src,dst)
 
     def copy(self,src,dst,**kwds):
         d = self._pre_copy(src,dst)
@@ -283,58 +395,64 @@ class WatchableFS(WrapFS,WatchableFSMixin):
         d = self._pre_copy(src,dst)
         super(WatchableFS,self).move(src,dst,**kwds)
         self._post_copy(src,dst,d)
-        self.notify_watchers(REMOVED,src)
+        self._post_move(src,dst,d)
 
     def movedir(self,src,dst,**kwds):
         d = self._pre_copy(src,dst)
         super(WatchableFS,self).movedir(src,dst,**kwds)
         self._post_copy(src,dst,d)
-        self.notify_watchers(REMOVED,src)
+        self._post_move(src,dst,d)
 
     def _pre_copy(self,src,dst):
         dst_paths = {}
         try:
             for (dirnm,filenms) in self.wrapped_fs.walk(dst):
-                dirnm = dirnm[len(dst):]
+                dirnm = dirnm[len(dst)+1:]
                 dst_paths[dirnm] = True
                 for filenm in filenms:
                     dst_paths[filenm] = False
         except ResourceNotFoundError:
             pass
         except ResourceInvalidError:
-            dst_paths[dst] = False
+            dst_paths[""] = False
         src_paths = {}
         try:
             for (dirnm,filenms) in self.wrapped_fs.walk(src):
-                dirnm = dirnm[len(src):]
+                dirnm = dirnm[len(src)+1:]
                 src_paths[dirnm] = True
                 for filenm in filenms:
                     src_paths[pathjoin(dirnm,filenm)] = False
         except ResourceNotFoundError:
             pass
         except ResourceInvalidError:
-            src_paths[src] = False
+            src_paths[""] = False
         return (src_paths,dst_paths)
 
     def _post_copy(self,src,dst,data):
         (src_paths,dst_paths) = data
-        for src_path,isdir in src_paths.iteritems():
+        for src_path,isdir in sorted(src_paths.items()):
             path = pathjoin(dst,src_path)
             if src_path in dst_paths:
                 self.notify_watchers(MODIFIED,path,not isdir)
             else:
                 self.notify_watchers(CREATED,path)
-        for dst_path,isdir in dst_paths.iteritems():
+        for dst_path,isdir in sorted(dst_paths.items()):
             path = pathjoin(dst,dst_path)
             if not self.wrapped_fs.exists(path):
                 self.notify_watchers(REMOVED,path)
 
+    def _post_move(self,src,dst,data):
+        (src_paths,dst_paths) = data
+        for src_path,isdir in sorted(src_paths.items(),reverse=True):
+            path = pathjoin(src,src_path)
+            self.notify_watchers(REMOVED,path)
+
     def setxattr(self,path,name,value):
         super(WatchableFS,self).setxattr(path,name,value)
         self.notify_watchers(MODIFIED,path,False)
 
     def delxattr(self,path,name):
-        super(WatchableFS,self).delxattr(path,name,value)
+        super(WatchableFS,self).delxattr(path,name)
         self.notify_watchers(MODIFIED,path,False)
 
 
@@ -342,7 +460,7 @@ class WatchableFS(WrapFS,WatchableFSMixin):
 class PollingWatchableFS(WatchableFS):
     """FS wrapper simulating watcher callbacks by periodic polling.
 
-    This FS wrapper augments the funcionality of WatchableFS by periodically
+    This FS wrapper augments the functionality of WatchableFS by periodically
     polling the underlying FS for changes.  It is thus capable of detecting
     changes made to the underlying FS via other interfaces, albeit with a
     (configurable) delay to account for the polling interval.
@@ -439,18 +557,18 @@ class PollingWatchableFS(WatchableFS):
                 for (k,v) in new_info.iteritems():
                     if k not in old_info:
                         was_modified = True
-                        break 
+                        break
                     elif old_info[k] != v:
                         if k in ("accessed_time","st_atime",):
                             was_accessed = True
                         elif k:
                             was_modified = True
-                            break 
+                            break
                 else:
                     for k in old_info:
                         if k not in new_info:
                             was_modified = True
-                            break 
+                            break
                 if was_modified:
                     self.notify_watchers(MODIFIED,fpath,True)
                 elif was_accessed:
@@ -472,12 +590,12 @@ def ensure_watchable(fs,wrapper_class=PollingWatchableFS,*args,**kwds):
     for watcher callbacks.  This may be the original object if it supports them
     natively, or a wrapper class if they must be simulated.
     """
+    if isinstance(fs,wrapper_class):
+        return fs
     try:
-        w = fs.add_watcher(lambda e: None,"/somepaththatsnotlikelytoexist")
-    except (AttributeError,UnsupportedError):
+        w = fs.add_watcher(lambda e: None,"/",recursive=False)
+    except (AttributeError,FSError):
         return wrapper_class(fs,*args,**kwds)
-    except FSError:
-        pass
     else:
         fs.del_watcher(w)
     return fs
