@@ -29,7 +29,7 @@ class LazyFS(WrapFS):
     """
 
     def __init__(self, fs):
-        super(LazyFS,self).__init__(fs)
+        super(LazyFS, self).__init__(fs)
         self._lazy_creation_lock = Lock()
 
     def __unicode__(self):
@@ -80,6 +80,8 @@ class LazyFS(WrapFS):
             self._fsclass = fs
             self._fsargs = []
             self._fskwds = {}
+        elif fs is None:
+            del self.__dict__['wrapped_fs']
         else:
             self._fsclass = fs[0]
             try:
